/*
 * Copyright (c) 2016, Freescale Semiconductor, Inc.
 * Copyright 2016-2017 NXP
 *
 * Redistribution and use in source and binary forms, with or without modification,
 * are permitted provided that the following conditions are met:
 *
 * o Redistributions of source code must retain the above copyright notice, this list
 *   of conditions and the following disclaimer.
 *
 * o Redistributions in binary form must reproduce the above copyright notice, this
 *   list of conditions and the following disclaimer in the documentation and/or
 *   other materials provided with the distribution.
 *
 * o Neither the name of Freescale Semiconductor, Inc. nor the names of its
 *   contributors may be used to endorse or promote products derived from this
 *   software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 * ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */


#ifndef _PIN_MUX_H_
#define _PIN_MUX_H_

/***********************************************************************************************************************
 * Definitions
 **********************************************************************************************************************/

/*! @brief Direction type  */
typedef enum _pin_mux_direction
{
    kPIN_MUX_DirectionInput = 0U,        /* Input direction */
    kPIN_MUX_DirectionOutput = 1U,       /* Output direction */
    kPIN_MUX_DirectionInputOrOutput = 2U /* Input or output direction */
} pin_mux_direction_t;

/*!
 * @addtogroup pin_mux
 * @{
 */

/***********************************************************************************************************************
 * API
 **********************************************************************************************************************/

#if defined(__cplusplus)
extern "C" {
#endif


/*!
 * @brief Select Analog/Digital mode.: Digital mode. */
#define PIO00_DIGIMODE_DIGITAL 0x01u
/*!
 * @brief Controls input glitch filter.: Filter disabled. No input filtering is done. */
#define PIO00_FILTEROFF_DISABLED 0x01u
/*!
 * @brief Selects pin function.: Alternative connection 1. */
#define PIO00_FUNC_ALT1 0x01u
/*!
 * @brief Input polarity.: Disabled. Input function is not inverted. */
#define PIO00_INVERT_DISABLED 0x00u
/*!
 * @brief Controls open-drain mode.: Normal. Normal push-pull output */
#define PIO00_OD_NORMAL 0x00u
/*!
 * @brief
 * Driver slew rate.
 * : Standard mode, output slew rate control is enabled.
 * More outputs can be switched simultaneously.
 */
#define PIO00_SLEW_STANDARD 0x00u
/*!
 * @brief Select Analog/Digital mode.: Digital mode. */
#define PIO01_DIGIMODE_DIGITAL 0x01u
/*!
 * @brief Controls input glitch filter.: Filter disabled. No input filtering is done. */
#define PIO01_FILTEROFF_DISABLED 0x01u
/*!
 * @brief Selects pin function.: Alternative connection 1. */
#define PIO01_FUNC_ALT1 0x01u
/*!
 * @brief Input polarity.: Disabled. Input function is not inverted. */
#define PIO01_INVERT_DISABLED 0x00u
/*!
 * @brief Controls open-drain mode.: Normal. Normal push-pull output */
#define PIO01_OD_NORMAL 0x00u
/*!
 * @brief
 * Driver slew rate.
 * : Standard mode, output slew rate control is enabled.
 * More outputs can be switched simultaneously.
 */
#define PIO01_SLEW_STANDARD 0x00u

/*!
 * @brief Configures pin routing and optionally pin electrical features.
 *
 */
void BOARD_InitPins(void); /* Function assigned for the Cortex-M0P */

#define IOCON_PIO_DIGITAL_EN 0x80u    /*!<@brief Enables digital function */
#define IOCON_PIO_FUNC1 0x01u         /*!<@brief Selects pin function 1 */
#define IOCON_PIO_FUNC2 0x02u         /*!<@brief Selects pin function 2 */
#define IOCON_PIO_INPFILT_OFF 0x0100u /*!<@brief Input filter disabled */
#define IOCON_PIO_INV_DI 0x00u        /*!<@brief Input function is not inverted */
#define IOCON_PIO_MODE_PULLUP 0x10u   /*!<@brief Selects pull-up function */
#define IOCON_PIO_OPENDRAIN_DI 0x00u  /*!<@brief Open drain is disabled */
#define IOCON_PIO_SLEW_STANDARD 0x00u /*!<@brief Standard mode, output slew rate control is enabled */

/*!
 * @brief Configures pin routing and optionally pin electrical features.
 *
 */
void SPI3_InitPins(void); /* Function assigned for the Cortex-M0P */

/*!
 * @brief Select Analog/Digital mode.: Digital mode. */
#define PIO011_DIGIMODE_DIGITAL 0x01u
/*!
 * @brief Controls input glitch filter.: Filter disabled. No input filtering is done. */
#define PIO011_FILTEROFF_DISABLED 0x01u
/*!
 * @brief Selects pin function.: Alternative connection 0. */
#define PIO011_FUNC_ALT0 0x00u
/*!
 * @brief Input polarity.: Disabled. Input function is not inverted. */
#define PIO011_INVERT_DISABLED 0x00u
/*!
 * @brief Controls open-drain mode.: Normal. Normal push-pull output */
#define PIO011_OD_NORMAL 0x00u
/*!
 * @brief Select Analog/Digital mode.: Digital mode. */
#define PIO012_DIGIMODE_DIGITAL 0x01u
/*!
 * @brief Controls input glitch filter.: Filter disabled. No input filtering is done. */
#define PIO012_FILTEROFF_DISABLED 0x01u
/*!
 * @brief Selects pin function.: Alternative connection 0. */
#define PIO012_FUNC_ALT0 0x00u
/*!
 * @brief Input polarity.: Disabled. Input function is not inverted. */
#define PIO012_INVERT_DISABLED 0x00u
/*!
 * @brief Controls open-drain mode.: Normal. Normal push-pull output */
#define PIO012_OD_NORMAL 0x00u
/*!
 * @brief Select Analog/Digital mode.: Digital mode. */
#define PIO013_DIGIMODE_DIGITAL 0x01u
/*!
 * @brief Controls input glitch filter.: Filter disabled. No input filtering is done. */
#define PIO013_FILTEROFF_DISABLED 0x01u
/*!
 * @brief Selects pin function.: Alternative connection 0. */
#define PIO013_FUNC_ALT0 0x00u
/*!
 * @brief Input polarity.: Disabled. Input function is not inverted. */
#define PIO013_INVERT_DISABLED 0x00u
/*!
 * @brief Controls open-drain mode.: Normal. Normal push-pull output */
#define PIO013_OD_NORMAL 0x00u
/*!
 * @brief Select Analog/Digital mode.: Digital mode. */
#define PIO04_DIGIMODE_DIGITAL 0x01u
/*!
 * @brief Controls input glitch filter.: Filter disabled. No input filtering is done. */
#define PIO04_FILTEROFF_DISABLED 0x01u
/*!
 * @brief Selects pin function.: Alternative connection 0. */
#define PIO04_FUNC_ALT0 0x00u
/*!
 * @brief Input polarity.: Disabled. Input function is not inverted. */
#define PIO04_INVERT_DISABLED 0x00u
/*!
 * @brief Controls open-drain mode.: Normal. Normal push-pull output */
#define PIO04_OD_NORMAL 0x00u

/*!
 * @brief Configures pin routing and optionally pin electrical features.
 *
 */
void SPI3_DeinitPins(void); /* Function assigned for the Cortex-M0P */

#define IOCON_PIO_DIGITAL_EN 0x80u    /*!<@brief Enables digital function */
#define IOCON_PIO_FUNC1 0x01u         /*!<@brief Selects pin function 1 */
#define IOCON_PIO_FUNC4 0x04u         /*!<@brief Selects pin function 4 */
#define IOCON_PIO_INPFILT_OFF 0x0100u /*!<@brief Input filter disabled */
#define IOCON_PIO_INV_DI 0x00u        /*!<@brief Input function is not inverted */
#define IOCON_PIO_MODE_PULLUP 0x10u   /*!<@brief Selects pull-up function */
#define IOCON_PIO_OPENDRAIN_DI 0x00u  /*!<@brief Open drain is disabled */
#define IOCON_PIO_SLEW_STANDARD 0x00u /*!<@brief Standard mode, output slew rate control is enabled */

/*!
 * @brief Configures pin routing and optionally pin electrical features.
 *
 */
void SPI5_InitPins(void); /* Function assigned for the Cortex-M0P */

#define PIO018_DIGIMODE_DIGITAL 0x01u /*!<@brief Select Analog/Digital mode.: Digital mode. */
#define PIO018_FUNC_ALT0 0x00u        /*!<@brief Selects pin function.: Alternative connection 0. */
#define PIO019_DIGIMODE_DIGITAL 0x01u /*!<@brief Select Analog/Digital mode.: Digital mode. */
#define PIO019_FUNC_ALT0 0x00u        /*!<@brief Selects pin function.: Alternative connection 0. */
#define PIO020_DIGIMODE_DIGITAL 0x01u /*!<@brief Select Analog/Digital mode.: Digital mode. */
#define PIO020_FUNC_ALT0 0x00u        /*!<@brief Selects pin function.: Alternative connection 0. */
#define PIO11_DIGIMODE_DIGITAL 0x01u  /*!<@brief Select Analog/Digital mode.: Digital mode. */
#define PIO11_FUNC_ALT0 0x00u         /*!<@brief Selects pin function.: Alternative connection 0. */

/*!
 * @brief Configures pin routing and optionally pin electrical features.
 *
 */
void SPI5_DeinitPins(void); /* Function assigned for the Cortex-M0P */

#if defined(__cplusplus)
}
#endif

/*!
 * @}
 */
#endif /* _PIN_MUX_H_ */

/***********************************************************************************************************************
 * EOF
 **********************************************************************************************************************/
